﻿// BallPhysicsGame Version 2 - Using Papervision 2.0 Beta - Revision 870, JibLibFlash - Revision 25
// A very simple physics based ball game, where you must knock off red squares. Level information is loaded via an xml file
//
// released under MIT License
// http://www.opensource.org/licenses/mit-license.php
//
//@author		Devin Reimer
//@version		1.0.1
//@website 		http://www.permaven.com, http://www.almostlogical.com


//Copyright (c) 2009 Devin Reimer

/*
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package com
{
	import flash.display.DisplayObject;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.KeyboardEvent;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	import flash.ui.Keyboard;
	import flash.events.IOErrorEvent;
	import flash.display.Loader;
	import flash.net.URLLoader;
	import flash.net.URLRequest;
	import flash.text.TextField;
	import flash.text.TextFieldAutoSize;
	import flash.geom.Point;
	
	import org.papervision3d.core.math.Number3D;
	import org.papervision3d.view.Viewport3D;
	import org.papervision3d.cameras.SpringCamera3D;
	import org.papervision3d.render.QuadrantRenderEngine;
	import org.papervision3d.materials.utils.MaterialsList;
	import org.papervision3d.materials.ColorMaterial;
	import org.papervision3d.materials.shadematerials.FlatShadeMaterial;
	import org.papervision3d.objects.primitives.Cube;
	import org.papervision3d.objects.primitives.Plane;
	import org.papervision3d.objects.primitives.Sphere;
	import org.papervision3d.objects.DisplayObject3D;
	import org.papervision3d.lights.PointLight3D;
	import org.papervision3d.scenes.Scene3D;
	import org.papervision3d.cameras.SpringCamera3D;

	import jiglib.math.*;
	import jiglib.geometry.*;
	import jiglib.physics.*;
	import jiglib.physics.constraint.*;


	public class BallPhysicsGame extends Sprite
	{
		private static const PLAYER_BALL_RADIUS:int = 25;
		private static const PLAYER_FORCE:int = 75;
		private static const PLAYER_WEIGHT:int = 5;
		private static const PLAYER_COLOUR:uint = 0xFBA248;
		
		private static const BASIC_GROUND_RESTITUTION:Number = 0.5;
		private static const BASIC_GROUND_STATIC_FRICTION:Number = 5;
		
		private static const WORLD_BOTTOM:int = -300;
		private static const STARTING_LEVEL:int = 1;
		private static const LEVEL_XML_PATH:String = "xml/AdvancedLevelInfo.xml";
		private static const MARKER_BOX_TYPE = "markerbox";	
		private var viewport:Viewport3D;
		private var scene:Scene3D;
		private var camera:SpringCamera3D;
		private var renderer:QuadrantRenderEngine;
	
		private var lightSource:PointLight3D;
		
		private var currentLevel:int = STARTING_LEVEL;
		private var keyRight:Boolean = false;
		private var keyLeft:Boolean = false;
		private var keyForward:Boolean = false;
		private var keyBack:Boolean = false;
		
		private var keyLeftCounter:int = 0;
		private var keyRightCounter:int = 0;
	
		private var xmlLoader:URLLoader
		
		private var playerBall:JSphere;
		private var playerCameraPosition:DisplayObject3D;
		private var cameraRotation:int = 0;
		private var isIntro:Boolean;
		private var introCounter:int;
	
		private var markerArray:Array;
		private var currentStatusTF:TextField;
		private var totalMarkers:int;
		
		private var resettableItemsArray:Array;
		private var resettableItemsPositionArray:Array;
		
		private var instructionMessage:TextField;
		private var instructionFormat:TextFormat;
		
		private var currentSpeed:int;
		
		private var playerResetNumber3D:Number3D = new Number3D();
		private var playerRotationY:int = 0;
		
	

		public function BallPhysicsGame()
		{
			stage.addEventListener(MouseEvent.CLICK, setup, false, 0, true); //wait until the user click to start
			
			graphics.beginFill(0xC9F1ED);
			graphics.drawRect(0, 0, stage.stageWidth, stage.stageHeight);
			graphics.endFill();
			instructionMessage = new TextField();
			instructionFormat  = new TextFormat(null, 35, 0x000000);
			instructionMessage.defaultTextFormat = instructionFormat;
			instructionMessage.text = "Click to Start Game";
			instructionMessage.autoSize = TextFieldAutoSize.LEFT;
			instructionMessage.x= (stage.stageWidth - instructionMessage.width) / 2;
			instructionMessage.y = (stage.stageHeight / 2) - 80;
			addChild(instructionMessage);
			stage.mouseChildren = false;
		}
		
		private function setup(evt:Event)
		{
			stage.removeEventListener(MouseEvent.CLICK, setup, false);
			removeChild(instructionMessage);
			stage.addEventListener( KeyboardEvent.KEY_DOWN, keyDownHandler );
			stage.addEventListener( KeyboardEvent.KEY_UP, keyUpHandler );
			isIntro = true; //is current in intro process
			introCounter = 0; //how many inc of rotation have occurred
			cameraRotation = 0;
			
			initPapervision3D();
			createPlayer();
			playerCameraPosition = new DisplayObject3D();
			camera.target = playerCameraPosition;
			createHUD();
			loadLevelXML(LEVEL_XML_PATH);
			
			//addChild(new FPS());
		}

		private function initPapervision3D():void
		{
			viewport = new Viewport3D(550,400);
			addChild(viewport);
			
			scene = new Scene3D();
			 
			lightSource = new PointLight3D(true, true);
			lightSource.x = 100;
			lightSource.y = 300;
			lightSource.z = -500;
					 			 
			camera = new SpringCamera3D();

			//camera.mass = 8;
			//camera.damping = 4;
			//camera.stiffness = 2;
			camera.mass = 6;
			camera.damping = 3;
			camera.stiffness = 1;
				
			camera.lookOffset = new Number3D(0, 20, 30);
			camera.positionOffset = new Number3D(0, 100, -270);
			
			camera.focus = 100;
			camera.zoom = 10;
			 
			renderer = new QuadrantRenderEngine(QuadrantRenderEngine.CORRECT_Z_FILTER);
		}
		
		private function createPlayer():void
		{
			var ballMaterial = new FlatShadeMaterial(lightSource,PLAYER_COLOUR);
			var ballSkin:Sphere = new Sphere(ballMaterial,PLAYER_BALL_RADIUS);
			
			scene.addChild(ballSkin);
			playerBall = new JSphere(ballSkin, true, PLAYER_BALL_RADIUS);
			playerBall.setMass(PLAYER_WEIGHT);
			resetPlayerPosition();
			PhysicsSystem.getInstance().AddBody(playerBall);
		}
		
		//loads in an xml to build the level
		private function buildLevelUsingXML(theXML:XML):void
		{
			var levelXML:XMLList;
			var groupNumber3D:Number3D;
			var groupName:String;
			
			levelXML = theXML.level.(@id == currentLevel);
			
			playerResetNumber3D = new Number3D(levelXML.@playerX, levelXML.@playerY, levelXML.@playerZ);
			playerRotationY = levelXML.@playerRotationY;
			resetPlayerPosition();
			
			markerArray = new Array();
			resettableItemsArray = new Array();
			resettableItemsPositionArray = new Array();
			for each (var groupXML:XML in levelXML.group) //goes through each group
			{
				groupNumber3D = new Number3D(groupXML.@x, groupXML.@y, groupXML.@z);
				groupName = groupXML.@name;
				
				for each (var physicsObjXML:XML in groupXML.physicsObj)
				{
					createObjFromXML(physicsObjXML,groupName,groupNumber3D);
				}
			}
			
			totalMarkers = markerArray.length;
			beginRender();
		}
		
		private function createObjFromXML(objXML:XML,groupName:String,groupNumber3D:Number3D)
		{
			var basicPropXML:XMLList = objXML.basicProp;
			var objMatList:MaterialsList = new MaterialsList();
			var objSkin:Cube;
			var objColour:uint = objXML.@colour;
			var objJBox:JBox;
			var objJBoxWidth:int = basicPropXML.@width;
			var objJBoxDepth:int = basicPropXML.@depth;
			var objJBoxHeight:int = basicPropXML.@height;
			var objJBoxNumber3D:Number3D = new Number3D(int(basicPropXML.@x) + (objJBoxWidth/2), int(basicPropXML.@y) - (objJBoxHeight/2), int(basicPropXML.@z) + (objJBoxDepth/2)); //this puts starting point at the top, back, left corner
			var objJBoxMass:int = objXML.@mass;
			var objType:String = objXML.@type;
			var objName:String = groupName + " - " + objXML.@name;
			var segmentsS:int = 1;
			var segmentsT:int = 1;
			var segmentsH:int = 1;
			var isMovable:Boolean = false;
			var isResettable:Boolean = false;
			
			objJBoxNumber3D.plusEq(groupNumber3D); //addes this to group
			
			if (objXML.@isMovable == "true")
			{
				isMovable = true;
			}
			if (objXML.@resettable == "true")
			{
				isResettable = true;
			}
				
			if (objJBoxWidth > 250 || objJBoxDepth > 250 || objJBoxHeight>250) //if object is large increase number of triangles
			{
				segmentsS = 3;
				segmentsT = 3;
				segmentsH = 3;
			}
		
			objMatList.addMaterial(new FlatShadeMaterial(lightSource,objColour),"all");
			objSkin = new Cube(objMatList, objJBoxWidth, objJBoxDepth, objJBoxHeight,segmentsS,segmentsT,segmentsH);
			
			scene.addChild(objSkin);
			
			objJBox = new JBox(objSkin, isMovable, objJBoxWidth, objJBoxDepth, objJBoxHeight);
			objJBox.MoveTo(new JNumber3D(objJBoxNumber3D.x, objJBoxNumber3D.y, objJBoxNumber3D.z), JMatrix3D.IDENTITY);
		
			if (isMovable)
			{
				if (objJBoxMass <= 0)
				{
					objJBoxMass = 1;
				}
				objJBox.setMass(objJBoxMass);
				if (objType == MARKER_BOX_TYPE)
				{
					markerArray.push(objJBox);
				}
				
				if (isResettable)
				{
					resettableItemsArray.push(objJBox);
					resettableItemsPositionArray.push(objJBoxNumber3D);
				}
			}
		
			PhysicsSystem.getInstance().AddBody(objJBox);
		}
		
		private function loadLevelXML(xmlFileName:String):void
		{
			xmlLoader = new URLLoader();
			xmlLoader.load(new URLRequest(xmlFileName));
			xmlLoader.addEventListener(Event.COMPLETE,onXMLLoadComplete,false,0,true);
			xmlLoader.addEventListener(IOErrorEvent.IO_ERROR, onLoadFailure,false,0,true);
		}
			
		private function onXMLLoadComplete(evt:Event):void
		{
			var theXML:XML;
			try
			{
				XML.ignoreWhitespace = true;
				theXML = XML(evt.target.data);
			}
			catch(error:Error)
			{
				trace("Error In XML (General)- " + error.message);
			}
			
			xmlLoader.removeEventListener(Event.COMPLETE,onXMLLoadComplete,false);
			xmlLoader.removeEventListener(IOErrorEvent.IO_ERROR, onLoadFailure, false);
			xmlLoader = null;
			
			buildLevelUsingXML(theXML);
			
		}
		private function onLoadFailure(evt:Event):void
		{
			trace("Load Failure: " + evt);
		}

		private function keyDownHandler(evt:KeyboardEvent):void
		{
			switch(evt.keyCode)
			{
				case Keyboard.UP:
					keyForward = true;
					keyBack = false;
					break;
				case Keyboard.DOWN:
					keyBack = true;
					keyForward = false;
					break;
				case Keyboard.LEFT:
					keyLeft = true;
					keyRight = false;
					break;
				case Keyboard.RIGHT:
					keyRight = true;
					keyLeft = false;
					break;
				case Keyboard.SPACE:
					trace(playerBall.CurrentState.Position.toString());
					break;
			}
		}

		private function keyUpHandler(evt:KeyboardEvent):void
		{
			switch(evt.keyCode)
			{
				case Keyboard.UP:
					keyForward = false;
					break;
				case Keyboard.DOWN:
					keyBack = false;
					break;
				case Keyboard.LEFT:
					keyLeft = false;
					break;
				case Keyboard.RIGHT:
					keyRight = false;
					break;
			}
		}
		
		private function checkPlayerPosition():void
		{
			if (playerBall.CurrentState.Position.y < WORLD_BOTTOM) //if fall off level reset layer
			{
				resetPlayerPosition();
			}
		}
		
		private function resetPlayerPosition():void
		{			
			playerBall.MoveTo(new JNumber3D(playerResetNumber3D.x, playerResetNumber3D.y + PLAYER_BALL_RADIUS, playerResetNumber3D.z),JMatrix3D.IDENTITY);
			cameraRotation = playerRotationY;
		}

		private function beginRender():void
		{
			updateHUDText();
			this.addEventListener(Event.ENTER_FRAME,render);
		}
		private function render(evt:Event):void
		{			
			if (isIntro && introCounter>50)
			{
				isIntro = false;
				removeChild(instructionMessage);
			}
			else if (isIntro)
			{
				introCounter++;
			}
			
			movePlayer();
			PhysicsSystem.getInstance().Integrate(0.1);
			checkPlayerPosition();
			checkMarkers();
			checkResettableItems();
			updateCamera();
	
			renderer.renderScene( scene, camera, viewport );
		}
		
		private function updateCamera():void
		{
			playerCameraPosition.copyPosition(JSphere(playerBall).BodySkin); //update camera position excluding angle
			playerCameraPosition.rotationY = cameraRotation;
		}
		
		private function movePlayer():void
		{
			var tempDisplayObject3D:DisplayObject3D  = new DisplayObject3D(); //used to calculate positions
			
			var maxStoppedRotationSpeed:int = 3.5;
			var currentRotationSpeed:int = 0;
			var tempInt:int;
			var inputRotationAngle:Number = 0; //used with to move camera if speed it not high enough
			var tractionRatio:Number = 1;
			
			tempDisplayObject3D.copyTransform(playerCameraPosition);
			tempDisplayObject3D.x = 0;
			tempDisplayObject3D.y = 0;
			tempDisplayObject3D.z = 0;
	
			if(keyLeft)
			{
				currentRotationSpeed = maxStoppedRotationSpeed;
				currentRotationSpeed /= 2;
				tempInt = keyLeftCounter/10;
				if (tempInt > currentRotationSpeed)
				{
					tempInt = currentRotationSpeed;
				}
				currentRotationSpeed += tempInt;
				if (currentSpeed >= 3)
				{
					currentRotationSpeed /= 2;
					tempDisplayObject3D.moveLeft(PLAYER_FORCE / 1.5);
				}
				else
				{
					tempDisplayObject3D.moveLeft(PLAYER_FORCE / 4);
				}
				inputRotationAngle-= currentRotationSpeed;
				keyLeftCounter++;
			}
			else
			{
				keyLeftCounter = 0;
			}
			if(keyRight)
			{
				currentRotationSpeed = maxStoppedRotationSpeed;
				currentRotationSpeed /= 2;
				tempInt = keyRightCounter/10;
				if (tempInt > currentRotationSpeed)
				{
					tempInt = currentRotationSpeed;
				}
				currentRotationSpeed += tempInt;
				if (currentSpeed >= 3)
				{
					currentRotationSpeed /= 2;
					tempDisplayObject3D.moveRight(PLAYER_FORCE/1.5);
				}
				else
				{
					tempDisplayObject3D.moveRight(PLAYER_FORCE/4);
				}
				inputRotationAngle+= currentRotationSpeed;
				keyRightCounter++;
			}
			else
			{
				keyRightCounter = 0;
			}
			if(keyForward)
			{
				tempDisplayObject3D.moveForward(PLAYER_FORCE);
			}
			if(keyBack)
			{
				tempDisplayObject3D.moveBackward(PLAYER_FORCE);
			}
			
			if (!hasTraction()) //if not touching object ball has no traction, so limit it movement
			{
				tractionRatio=3;
			}
			playerBall.AddWorldForce(new JNumber3D(tempDisplayObject3D.x/tractionRatio, tempDisplayObject3D.y/tractionRatio, tempDisplayObject3D.z/tractionRatio), playerBall.CurrentState.Position);
			

			updateCameraRotation(inputRotationAngle);
		}
		
		
		//inputRotationAngle - if speed is too low use inputRotationAngle for camera moment
		private function updateCameraRotation(inputRotationAngle:Number):void
		{
			var angleDegrees:Number;
			var differenceBetweenAngles:Number;
			var angleDistance:Number;
			var angleToMove:Number;
			var reverseDeadZone:int = 20;
			var angleDirection:int = 0; // +1, 0, -1
			
			setCurrentSpeed();
			
			angleDegrees = getAngleOfMotionInPapervisionAngleSystem();
			
			cameraRotation = cameraRotation % 360;
			if (cameraRotation < 0)
			{
				cameraRotation += 360;
			}
			
			differenceBetweenAngles = cameraRotation - angleDegrees;
			angleDistance = differenceBetweenAngles % (180+1); //180 + 1 so don't round furthest angle
			
			if (angleDistance < 0) { angleDistance *= -1; }
			
			if ((differenceBetweenAngles > 0 && differenceBetweenAngles < 180 - reverseDeadZone) || differenceBetweenAngles < -180 - reverseDeadZone)
			{	//move in negative direction
				angleDirection = -1;
			}
			else if ((differenceBetweenAngles < 0 && differenceBetweenAngles > -180 + reverseDeadZone) || differenceBetweenAngles > 180 + reverseDeadZone)
			{	//move in positive direction
				angleDirection = 1;	
			}
			else
			{	//don't move angle (currently in a dead zone)
				angleDirection = 0;
			}
				
			if (currentSpeed >= 1) //make sure current speed is used before overriding camera motion from left right keys
			{			
				if (angleDistance > 1)
				{
					angleToMove = angleDistance / (20.0 - (currentSpeed));
					cameraRotation += (angleToMove * angleDirection);
				}
			}
			else //if not moving quick enough use arrow keys to determine rotation
			{
				cameraRotation += inputRotationAngle;
			}
			
			cameraRotation %= 360;	
		}
		
		//if ball has traction with another object
		private function hasTraction():Boolean
		{
			return JSphere(playerBall).Collisions.length > 0;
		}
		
		private function setCurrentSpeed():void
		{
			var newJNumber:JNumber3D = JSphere(playerBall).CurrentState.Position;
			var oldJNumber:JNumber3D = JSphere(playerBall).OldState.Position;
			var oldPosition:DisplayObject3D = new DisplayObject3D(); //used to keep track of old position
			var newPosition:DisplayObject3D = new DisplayObject3D(); //used to keep track of new position
	
			oldPosition.position = new Number3D(newJNumber.x, newJNumber.y, newJNumber.z);
			newPosition.position = new Number3D(oldJNumber.x, oldJNumber.y, oldJNumber.z);
			
			currentSpeed = newPosition.distanceTo(oldPosition);
		}
		
		//as the rotation angles in papervision are different from standard angle system, covert
		// also only calculate angle on x and z axis
		private function getAngleOfMotionInPapervisionAngleSystem():Number
		{
			var newJNumber:JNumber3D = JSphere(playerBall).CurrentState.Position;
			var oldJNumber:JNumber3D = JSphere(playerBall).OldState.Position;
			var dx:Number = (newJNumber.x - oldJNumber.x);
			var dz:Number = (newJNumber.z - oldJNumber.z);
			var angleRadians:Number = Math.atan2(dz,dx);
			var angleDegrees:Number = angleRadians * 180 / Math.PI;
			
			//better formula for this calculation, fix later
			angleDegrees -= 360;
			angleDegrees *= -1;
			angleDegrees += 90;
			angleDegrees %= 360;
			
			return angleDegrees;
		}
		
		//check to see if any markers have been successfully pushed off the edge
		private function checkMarkers()
		{
			var markerObj:JBox;
			var isUpdate:Boolean = false;
			
			for (var counter:int = 0; counter < markerArray.length; counter++)
			{
				markerObj = markerArray[counter];
				if (markerObj != null)
				{
					if (markerObj.CurrentState.Position.y < WORLD_BOTTOM) 
					{
						PhysicsSystem.getInstance().RemoveBody(markerObj);
						scene.removeChild(markerObj.BodySkin);
						markerArray.splice(markerArray.indexOf(markerObj), 1);
						counter--;
						markerObj = null;
						isUpdate = true;
					}
				}
			}
			
			if (isUpdate)
			{
				updateHUDText();
			}
		}
		
		//check if any items that are resettable have fallen off world, if they have put back to orginal location
		private function checkResettableItems()
		{
			var resettableItem:JBox;
			var resettablePosition:Number3D;
			
			for (var counter:int = 0; counter < resettableItemsArray.length; counter++)
			{
				resettableItem = resettableItemsArray[counter];
				if (resettableItem != null)
				{
					if (resettableItem.CurrentState.Position.y < WORLD_BOTTOM) 
					{
						resettablePosition = resettableItemsPositionArray[counter];
						resettableItem.MoveTo(new JNumber3D(resettablePosition.x, resettablePosition.y, resettablePosition.z), JMatrix3D.IDENTITY);
					}
				}
			}
			
		}
		
		private function createHUD()
		{
			var backgroundSprite:Sprite = new Sprite();
			var blogNameTF:TextField;
			
			backgroundSprite.graphics.beginFill(0xFFFFFF);
			backgroundSprite.graphics.drawRect(0, 0, 200, 20);
			backgroundSprite.graphics.endFill();
			
			currentStatusTF = new TextField();
		
			currentStatusTF.text = "Loading...";
			currentStatusTF.textColor = 0xF70000;
			currentStatusTF.autoSize = TextFieldAutoSize.LEFT;
			currentStatusTF.x = 20;
			currentStatusTF.y = 2;
			currentStatusTF.width = 200;
			backgroundSprite.addChild(currentStatusTF);
			
			backgroundSprite.x = (stage.stageWidth - backgroundSprite.width);
			addChild(backgroundSprite);
			
			instructionMessage = new TextField();
			instructionFormat  = new TextFormat(null, 35, 0xF70000);
			instructionFormat.align = TextFormatAlign.CENTER;
			instructionMessage.defaultTextFormat = instructionFormat;
			instructionMessage.text = "Knock off all red boxes to win!\nUse Arrow Keys to Play";
			instructionMessage.autoSize = TextFieldAutoSize.LEFT;
			instructionMessage.x= (stage.stageWidth - instructionMessage.width) / 2;
			//instructionMessage.width = stage.stageWidth;
			instructionMessage.y = (stage.stageHeight / 2)-80;
			addChild(instructionMessage);
			
			blogNameTF = new TextField();
			blogNameTF.text = "http://blog.almostlogical.com - Devin Reimer";
			blogNameTF.textColor = 0xFFFFFF;
			blogNameTF.autoSize = TextFieldAutoSize.LEFT;
			blogNameTF.x = (stage.stageWidth - blogNameTF.width);
			blogNameTF.y = (stage.stageHeight - blogNameTF.height);
			addChild(blogNameTF);
		}
		
		private function updateHUDText()
		{
			if (markerArray.length == 0)
			{
				currentStatusTF.text = "Winner!!!";
				instructionMessage.text = "Winner!!!";
				instructionMessage.autoSize = TextFieldAutoSize.LEFT;
				instructionMessage.x= (stage.stageWidth - instructionMessage.width) / 2;
				addChild(instructionMessage);
			}
			else
			{
				currentStatusTF.text = "Red Boxes Remaining: " + markerArray.length + " out of " + totalMarkers;
			}
		}
	}//end class BallPhysics
}